@@ -11,7 +11,7 @@
  *  You may distribute this under the terms of either the GNU General Public
  *  License or the Artistic License, as specified in the Perl README file.
  *
- *  $Id: dbdimp.h,v 1.16 2005/04/13 02:22:41 capttofu Exp $
+ *  $Id: dbdimp.h,v 1.5.4.7 2005/04/22 22:27:31 capttofu Exp $
  */
 
 /*
@@ -19,8 +19,16 @@
  */
 #include <DBIXS.h>  /* installed by the DBI module                        */
 #include <mysql.h>  /* Comes with MySQL-devel */
+#include <mysqld_error.h>  /* Comes MySQL */
 #include <errmsg.h> /* Comes with MySQL-devel */
 
+/*
+ * This is the version of MySQL wherer
+ * the server will be used to process prepare
+ * statements as opposed to emulation in the driver
+*/
+#define SERVER_PREPARE_VERSION 40103
+
 /*
  *  The following are return codes passed in $h->err in case of
  *  errors by DBD::mysql.
@@ -45,6 +53,7 @@ enum errMsgs {
     JW_ERR_MEM,
     JW_ERR_LIST_INDEX,
     JW_ERR_SEQUENCE,
+    AS_ERR_EMBEDDED,
     TX_ERR_AUTOCOMMIT,
     TX_ERR_COMMIT,
     TX_ERR_ROLLBACK
@@ -85,8 +94,17 @@ enum av_attribs {
  *  This declares a variable called "imp_drh" of type
  *  "struct imp_drh_st *".
  */
+typedef struct imp_drh_embedded_st {
+    int state;
+    SV * args;
+    SV * groups;
+} imp_drh_embedded_t;
+
 struct imp_drh_st {
     dbih_drc_t com;         /* MUST be first element in structure   */
+#if defined(DBD_MYSQL_EMBEDDED)
+    imp_drh_embedded_t embedded;     /* */
+#endif
 };
 
 
@@ -102,7 +120,7 @@ struct imp_drh_st {
  */
 struct imp_dbh_st {
     dbih_dbc_t com;         /*  MUST be first element in structure   */
-    
+
     MYSQL mysql;
     int has_transactions;   /*  boolean indicating support for
 			     *  transactions, currently always
@@ -115,11 +133,15 @@ struct imp_dbh_st {
 	    unsigned int auto_reconnects_failed;
     } stats;
     unsigned short int  bind_type_guessing;
+    int use_mysql_use_result; /* TRUE if execute should use
+                               * mysql_use_result rather than
+                               * mysql_store_result
+                               */
+    int use_server_side_prepare;
+    int has_autodetect_prepare;
 };
 
 
-
-
 /*
  *  The bind_param method internally uses this structure for storing
  *  parameters.
@@ -129,6 +151,37 @@ typedef struct imp_sth_ph_st {
     int type;
 } imp_sth_ph_t;
 
+/*
+ *  The bind_param method internally uses this structure for storing
+ *  parameters.
+ */
+typedef struct imp_sth_phb_st {
+    unsigned long   length;
+    char            is_null;
+} imp_sth_phb_t;
+
+/*
+ *  The dbd_describe uses this structure for storing
+ *  fields meta info.
+ *  Added ddata, ldata, lldata for accomodate
+ *  being able to use different data types
+ *  12.02.20004 PMG
+ */
+typedef struct imp_sth_fbh_st {
+    unsigned long  length;
+    bool           is_null;
+    char           * data;
+    double        ddata;
+    long          ldata;
+    long long     lldata;
+
+} imp_sth_fbh_t;
+
+
+typedef struct imp_sth_fbind_st {
+   unsigned long   * length;
+   char            * is_null;
+} imp_sth_fbind_t;
 
 
 /*
@@ -144,9 +197,20 @@ typedef struct imp_sth_ph_st {
 struct imp_sth_st {
     dbih_stc_t com;       /* MUST be first element in structure     */
 
-    MYSQL_RES* cda;       /* result                                 */
+#if (MYSQL_VERSION_ID >= SERVER_PREPARE_VERSION)
+    MYSQL_STMT       *stmt;
+    MYSQL_BIND       *bind;
+    MYSQL_BIND       *buffer;
+    imp_sth_phb_t    *fbind;
+    imp_sth_fbh_t    *fbh;
+    int              has_been_bound;
+    int use_server_side_prepare;     /* does server support new binary protocol */
+#endif
+
+    MYSQL_RES* result;       /* result                                 */
     int currow;           /* number of current row                  */
-    my_ulonglong row_num; /* total number of rows                   */
+    int fetch_done;       /* mark that fetch done                   */
+    my_ulonglong row_num;         /* total number of rows                   */
 
     int   done_desc;      /* have we described this sth yet ?	    */
     long  long_buflen;    /* length for long/longraw (if >0)	    */
@@ -163,8 +227,7 @@ struct imp_sth_st {
 /*
  *  And last, not least: The prototype definitions.
  *
- * These defines avoid name clashes for multiple statically linked DBD's
- */
+ * These defines avoid name clashes for multiple statically linked DBD's	*/
 #define dbd_init		mysql_dr_init
 #define dbd_db_login		mysql_db_login
 #define dbd_db_do		mysql_db_do
@@ -192,10 +255,8 @@ struct imp_sth_st {
 #define dbd_db_type_info_all    mysql_db_type_info_all
 #define dbd_db_quote            mysql_db_quote
 
-#ifdef DBD_MYSQL_INSERT_ID_IS_GOOD /* prototype was broken in 
-				      some versions of dbi */
+#ifdef DBD_MYSQL_INSERT_ID_IS_GOOD /* prototype was broken in some versions of dbi */
 #define dbd_db_last_insert_id   mysql_db_last_insert_id
-
 #endif
 
 #include <dbd_xsh.h>
@@ -205,13 +266,26 @@ SV	*dbd_db_fieldlist (MYSQL_RES* res);
 void    dbd_preparse (imp_sth_t *imp_sth, SV *statement);
 my_ulonglong mysql_st_internal_execute(SV*, SV*, SV*, int, imp_sth_ph_t*, MYSQL_RES**,
 			      MYSQL*, int);
+
+#if MYSQL_VERSION_ID >= SERVER_PREPARE_VERSION
+my_ulonglong mysql_st_internal_execute41(SV*, SV*, SV*, int, imp_sth_ph_t*, MYSQL_RES**,
+                              MYSQL*, int, MYSQL_STMT*, MYSQL_BIND*, int*);
+
+int mysql_st_clean_cursor(SV*, imp_sth_t*);
+#endif
+
+#if defined(DBD_MYSQL_EMBEDDED)
+int count_embedded_options(char *);
+char ** fill_out_embedded_options(char *, int , int , int );
+int free_embedded_options(char **, int);
+/* We have to define dbd_discon_all method for mysqlEmb driver at least
+   to be able to stop embedded server properly */
+#define dbd_discon_all dbd_discon_all
+#endif
+
 AV* dbd_db_type_info_all (SV* dbh, imp_dbh_t* imp_dbh);
 SV* dbd_db_quote(SV*, SV*, SV*);
-extern MYSQL* mysql_dr_connect(MYSQL*, char*, char*, char*, char*, char*,
+extern MYSQL* mysql_dr_connect(SV*, MYSQL*, char*, char*, char*, char*, char*,
 			       char*, imp_dbh_t*);
 
-
 extern int mysql_db_reconnect(SV*);
-
-/* SV *mysql_db_last_insert_id(SV* dbh, imp_dbh_t *imp_dbh,
-        SV *catalog, SV *schema, SV *table, SV *field,SV *attr); */
